==> meteor-react-materializecss-1.0.33.tgz: package/src/components/form/useForm.jsx <==
import {Meteor} from "meteor/meteor";
import React, {useState} from 'react';
import _ from 'lodash';
import M from 'materialize-css';
import {FlowRouter} from 'meteor/ostrio:flow-router-extra';
import {i18n} from 'meteor/universe:i18n';
import { setUnsavedChanges } from "./preventUnsavedChanges";


/**
 * return input value - objekt {name, value}
 *
 * @param e
 * @returns {{name, _value: null}}
 */
export const getFormInputValue = (e) => {
    const {name, value, type, checked, multiple, tagName} = e.currentTarget;
    let _value = value || null;
    if (tagName && tagName.toLowerCase() === 'select' && multiple) {
        //Select multiple
        const elem = e.currentTarget;
        const instance = M.FormSelect.getInstance(elem)
        _value = instance.getSelectedValues();
    } else if (type === 'checkbox') {
        _value = checked;
    }
    return {name, _value};
}

/**
 * save via Meteor.call, show toast and return to list if there is no error
 *
 * @param id
 * @param values
 * @param insertMethod
 * @param updateMethod
 * @param successInsertMessage
 * @param successUpdateMessage
 * @param backRoute
 * @param backRouteParams
 */
export const callFormUpdateMethod = (
    id, values,
    insertMethod, updateMethod,
    successInsertMessage, successUpdateMessage,
    backRoute, backRouteParams = {}
) => {
    M.Toast.dismissAll();
    if (id) {
        Meteor.call(updateMethod, id, values, (e, r) => {
            if (e) {
                e = e || i18n.__('messages.error');
                const message = e.error || e.message || e;
                console.error(e);
                M.toast({
                    html: i18n.__('messages.save_error') + ': ' + message,
                    classes: 'text-danger'
                })
            } else {
                M.toast({
                    html: successUpdateMessage,
                    classes: 'text-success'
                });
                setUnsavedChanges(false);
                if (backRoute) {
                    FlowRouter.go(backRoute, backRouteParams);
                }
            }
        });
    } else {
        Meteor.call(insertMethod, values, (e, r) => {
            if (e) {
                e = e || i18n.__('messages.error');
                const message = e.error || e.message || e;
                console.error(e);
                M.toast({
                    html: i18n.__('messages.save_error') + ': ' + message,
                    classes: 'text-danger'
                });
            } else {
                M.toast({
                    html: successInsertMessage,
                    classes: 'text-success'
                });
                setUnsavedChanges(false);
                if (backRoute) {
                    if (r){
                        //meteor method must return _id
                        backRouteParams.editId = r;
                    }
                    FlowRouter.go(backRoute, backRouteParams);
                }
            }
        });
    }
};


/**
 * validate by schema,
 * return errors as objekt - equals values
 * for values do schema.clean
 *
 * @param _values
 * @param schema
 * @returns {{}}
 */
export const validateFormValues = (_values, schema) => {
    const _errors = {};
    try {
        _values = schema.clean(_values);
        schema.validate(_values);
    } catch (err) {
        if (err.details) {
            err.details.forEach(e => {
                _.set(_errors, e.name, e.message);
            });
        }
    }
    return _errors;
};

const checkChangedValue = (newValue, oldValue) => {
    if (JSON.stringify(newValue) !== JSON.stringify(oldValue)){
        setUnsavedChanges(true);
    }
};

const useForm = (defaultValues, schema, submit) => {
    const validate = (_values) => {
        return validateFormValues(_values, schema);
    }
    const [values, setValues] = useState(defaultValues);
    const [errors, setErrors] = useState(validate(defaultValues));

    //exactly set value a validate by scheme, name maybe with dot
    const setValue = (name, _value) => {
        checkChangedValue(values[name], _value);
        const _values = {...values};
        _.set(_values, name, _value);
        const _errors = validate(_values);
        setValues(_values);
        setErrors(_errors);
        
    }
    const handleChange = (e) => {
        if (e && e.preventDefault) {
            //e.preventDefault();
        }
        const {name, _value} = getFormInputValue(e);
        setValue(name, _value);
    }
    const handleSubmit = (e) => {
        if (e) {
            e.preventDefault();
        }
        const _errors = validate(values);
        setErrors(_errors);
        if (Object.keys(_errors).length === 0) {
            submit(values);
        }
    }
    const hasError = () => {
        const _errors = validate(values);
        return Object.keys(_errors).length > 0 ? true : false
    }

    const updateValues = (newValues) => {
        const _values = {...values, ...newValues};
        checkChangedValue(values, _values);
        const _errors = validate(_values);
        setValues(_values);
        setErrors(_errors);
        
    }

    return {values, updateValues, setValue, errors, hasError, handleChange, handleSubmit}
}

export default useForm;

==> meteor-react-materializecss-1.0.36.tgz: package/src/components/form/useForm.jsx <==
import {Meteor} from "meteor/meteor";
import React, {useState} from 'react';
import _ from 'lodash';
import M from 'materialize-css';
import {FlowRouter} from 'meteor/ostrio:flow-router-extra';
import {i18n} from 'meteor/universe:i18n';
import { setUnsavedChanges } from "./preventUnsavedChanges";


/**
 * return input value - objekt {name, value}
 *
 * @param e
 * @returns {{name, _value: null}}
 */
export const getFormInputValue = (e) => {
    const {name, value, type, checked, multiple, tagName} = e.currentTarget;
    let _value = value || null;
    if (tagName && tagName.toLowerCase() === 'select' && multiple) {
        //Select multiple
        const elem = e.currentTarget;
        const instance = M.FormSelect.getInstance(elem)
        _value = instance.getSelectedValues();
    } else if (type === 'checkbox') {
        _value = checked;
    }
    return {name, _value};
}

/**
 * save via Meteor.call, show toast and return to list if there is no error
 *
 * @param id
 * @param values
 * @param insertMethod
 * @param updateMethod
 * @param successInsertMessage
 * @param successUpdateMessage
 * @param backRoute
 * @param backRouteParams
 */
export const callFormUpdateMethod = (
    id, values,
    insertMethod, updateMethod,
    successInsertMessage, successUpdateMessage,
    backRoute, backRouteParams = {}
) => {
    M.Toast.dismissAll();
    if (id) {
        Meteor.call(updateMethod, id, values, (e, r) => {
            if (e) {
                e = e || i18n.__('messages.error');
                const message = e.error || e.message || e;
                console.error(e);
                M.toast({
                    html: i18n.__('messages.save_error') + ': ' + message,
                    classes: 'text-danger'
                })
            } else {
                M.toast({
                    html: successUpdateMessage,
                    classes: 'text-success'
                });
                setUnsavedChanges(false);
                if (backRoute) {
                    FlowRouter.go(backRoute, backRouteParams);
                }
            }
        });
    } else {
        Meteor.call(insertMethod, values, (e, r) => {
            if (e) {
                e = e || i18n.__('messages.error');
                const message = e.error || e.message || e;
                console.error(e);
                M.toast({
                    html: i18n.__('messages.save_error') + ': ' + message,
                    classes: 'text-danger'
                });
            } else {
                M.toast({
                    html: successInsertMessage,
                    classes: 'text-success'
                });
                setUnsavedChanges(false);
                if (backRoute) {
                    if (r){
                        //meteor method must return _id
                        backRouteParams.editId = r;
                    }
                    FlowRouter.go(backRoute, backRouteParams);
                }
            }
        });
    }
};


/**
 * validate by schema,
 * return errors as objekt - equals values
 * for values do schema.clean
 *
 * @param _values
 * @param schema
 * @returns {{}}
 */
export const validateFormValues = (_values, schema) => {
    const _errors = {};
    try {
        _values = schema.clean(_values);
        schema.validate(_values);
    } catch (err) {
        if (err.details) {
            err.details.forEach(e => {
                _.set(_errors, e.name, e.message);
            });
        }
    }
    return _errors;
};

const checkChangedValue = (newValue, oldValue) => {
    if (JSON.stringify(newValue) !== JSON.stringify(oldValue)){
        setUnsavedChanges(true);
    }
};

const useForm = (defaultValues, schema, submit, saveChanges=true) => {
    const validate = (_values) => {
        return validateFormValues(_values, schema);
    }
    const [values, setValues] = useState(defaultValues);
    const [errors, setErrors] = useState(validate(defaultValues));

    //exactly set value a validate by scheme, name maybe with dot
    const setValue = (name, _value) => {
        if (saveChanges){
            checkChangedValue(values[name], _value);
        }
        const _values = {...values};
        _.set(_values, name, _value);
        const _errors = validate(_values);
        setValues(_values);
        setErrors(_errors);
        
    }
    const handleChange = (e) => {
        if (e && e.preventDefault) {
            //e.preventDefault();
        }
        const {name, _value} = getFormInputValue(e);
        setValue(name, _value);
    }
    const handleSubmit = (e) => {
        if (e) {
            e.preventDefault();
        }
        const _errors = validate(values);
        setErrors(_errors);
        if (Object.keys(_errors).length === 0) {
            submit(values);
        }
    }
    const hasError = () => {
        const _errors = validate(values);
        return Object.keys(_errors).length > 0 ? true : false
    }

    const updateValues = (newValues) => {
        const _values = {...values, ...newValues};
        if (saveChanges){
            checkChangedValue(values, _values);
        }
        const _errors = validate(_values);
        setValues(_values);
        setErrors(_errors);
        
    }

    return {values, updateValues, setValue, errors, hasError, handleChange, handleSubmit}
}

export default useForm;
